import 'package:bytedesk_kefu/model/message.dart';
import 'package:bytedesk_kefu/model/uploadJsonResult.dart';
import 'package:equatable/equatable.dart';
import 'package:meta/meta.dart';

@immutable
abstract class MessageState extends Equatable {
  const MessageState();

  @override
  List<Object> get props => [];
}

class InitialMessageState extends MessageState {}

class MessageLoading extends MessageState {
  @override
  String toString() => 'MessageLoading';
}

class MessageUpLoading extends MessageState {
  @override
  String toString() => 'MessageUpLoading';
}

class ReceiveMessageState extends MessageState {
  final Message message;

  ReceiveMessageState({@required this.message}) : super();
}

class UploadImageSuccess extends MessageState {
  final UploadJsonResult uploadJsonResult;

  const UploadImageSuccess(this.uploadJsonResult);

  @override
  List<Object> get props => [uploadJsonResult];

  @override
  String toString() => 'UploadImageSuccess { logo: ${uploadJsonResult.url} }';
}

class UpLoadImageError extends MessageState {
  @override
  String toString() => 'UpLoadImageError';
}

class UploadVideoSuccess extends MessageState {
  final UploadJsonResult uploadJsonResult;

  const UploadVideoSuccess(this.uploadJsonResult);

  @override
  List<Object> get props => [uploadJsonResult];

  @override
  String toString() => 'UploadVideoSuccess { logo: ${uploadJsonResult.url} }';
}

class UpLoadVideoError extends MessageState {
  @override
  String toString() => 'UpLoadVideoError';
}

class QueryAnswerSuccess extends MessageState {
  final Message query;
  final Message answer;

  QueryAnswerSuccess({@required this.query, @required this.answer}) : super();
}

class MessageAnswerSuccess extends MessageState {
  final Message query;
  final Message answer;

  MessageAnswerSuccess({@required this.query, @required this.answer}) : super();
}

class RateAnswerSuccess extends MessageState {
  final Message result;

  RateAnswerSuccess({@required this.result}) : super();
}
