import 'package:bytedesk_kefu/model/user.dart';
import 'package:equatable/equatable.dart';
import 'package:meta/meta.dart';

@immutable
abstract class ProfileState extends Equatable {
  // ProfileState([List props = const []]) : super(props);
  const ProfileState();

  @override
  List<Object> get props => [];
}

class InitialProfileState extends ProfileState {}

class ProfileInProgress extends ProfileState {
  @override
  String toString() => 'ProfileInProgress';
}

class ProfileSuccess extends ProfileState {
  final User user;

  ProfileSuccess({@required this.user});

  @override
  String toString() => 'ProfileSuccess';
}

class ProfileError extends ProfileState {
  @override
  String toString() => 'ProfileError';
}

class ProfileFailure extends ProfileState {
  final String error;

  const ProfileFailure({@required this.error});

  @override
  List<Object> get props => [error];

  @override
  String toString() => 'ProfileFailure { error: $error }';
}

class UploadImageSuccess extends ProfileState {
  final String url;

  const UploadImageSuccess(this.url);

  @override
  List<Object> get props => [url];

  @override
  String toString() => 'UploadImageSuccess { logo: $url }';
}

class UpLoadImageError extends ProfileState {
  @override
  String toString() => 'UpLoadImageError';
}

class UpdateAvatarSuccess extends ProfileState {
  final User user;

  const UpdateAvatarSuccess(this.user);

  @override
  List<Object> get props => [user];

  @override
  String toString() => 'UpdateAvatarSuccess';
}

class UpdateNicknameSuccess extends ProfileState {
  final User user;

  const UpdateNicknameSuccess(this.user);

  @override
  List<Object> get props => [user];

  @override
  String toString() => 'UpdateNicknameSuccess';
}

class UpdateDescriptionSuccess extends ProfileState {
  final User user;

  const UpdateDescriptionSuccess(this.user);

  @override
  List<Object> get props => [user];

  @override
  String toString() => 'UpdateDescriptionSuccess';
}

class UpdateError extends ProfileState {
  @override
  String toString() => 'UpdateError';
}
