import 'package:bytedesk_kefu/util/bytedesk_constants.dart';
import 'package:bytedesk_kefu/util/bytedesk_utils.dart';
import 'package:equatable/equatable.dart';

class Thread extends Equatable {
  final String tid;
  final String topic;
  final String wid;
  final String nickname;
  final String avatar;
  final String content;
  final String timestamp;
  final int unreadCount;
  final String type;
  final String client;

  const Thread(
      {this.tid,
      this.topic,
      this.wid,
      this.nickname,
      this.avatar,
      this.content,
      this.timestamp,
      this.unreadCount,
      this.type, 
      this.client});

  static Thread fromWorkGroupJson(dynamic json) {
    return Thread(
        tid: json['tid'],
        topic: json['topic'],
        wid: json['workGroup']['wid'],
        nickname: json['workGroup']['nickname'],
        avatar: json['workGroup']['avatar'],
        content: json['content'],
        timestamp: BytedeskUtils.getTimeDuration(json['timestamp']),
        unreadCount: json['unreadCount'],
        type: json['type']);
  }

  static Thread fromWorkGroupJson2(dynamic json) {

    return Thread(
        tid: json['tid'],
        topic: json['topic'],
        wid: json['type'] == BytedeskConstants.THREAD_TYPE_WORKGROUP ? json['workGroup']['wid'] : ( json['type'] == BytedeskConstants.THREAD_TYPE_APPOINTED ? json['agent']['uid'] : json['admin']['uid'] ),
        nickname: json['type'] == BytedeskConstants.THREAD_TYPE_WORKGROUP ? json['workGroup']['nickname'] : ( json['type'] == BytedeskConstants.THREAD_TYPE_APPOINTED ? json['agent']['nickname'] : json['admin']['nickname'] ),
        avatar: json['type'] == BytedeskConstants.THREAD_TYPE_WORKGROUP ? json['workGroup']['avatar'] : ( json['type'] == BytedeskConstants.THREAD_TYPE_APPOINTED ? json['agent']['avatar'] : json['admin']['avatar'] ),
        content: json['content'],
        timestamp: json['timestamp'],
        unreadCount: json['unreadCount'],
        type: json['type']);
  }

  static Thread fromHistoryJson(dynamic json) {
    return Thread(
        tid: json['tid'],
        topic: json['topic'],
        // wid: json['wid'],
        nickname: json['nickname'],
        avatar: json['avatar'],
        content: json['content'],
        timestamp: BytedeskUtils.getTimeDuration(json['timestamp']),
        unreadCount: json['unreadCount'],
        type: json['type']);
  }

  static Thread fromVisitorJson(dynamic json) {
    return Thread(
        tid: json['tid'],
        topic: json['topic'],
        nickname: json['visitor']['nickname'],
        avatar: json['visitor']['avatar'],
        content: json['content'],
        timestamp: BytedeskUtils.getTimeDuration(json['timestamp']),
        unreadCount: json['unreadCount'],
        type: json['type']);
  }

  static Thread fromContactJson(dynamic json) {
    return Thread(
        tid: json['tid'],
        topic: json['topic'],
        nickname: json['contact']['nickname'],
        avatar: json['contact']['avatar'],
        content: json['content'],
        timestamp: BytedeskUtils.getTimeDuration(json['timestamp']),
        unreadCount: json['unreadCount'],
        type: json['type']);
  }

  static Thread fromGroupJson(dynamic json) {
    return Thread(
        tid: json['tid'],
        topic: json['topic'],
        nickname: json['group']['nickname'],
        avatar: json['group']['avatar'],
        content: json['content'],
        timestamp: BytedeskUtils.getTimeDuration(json['timestamp']),
        unreadCount: json['unreadCount'],
        type: json['type']);
  }

  @override
  List<Object> get props => [topic];
}
